/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "voltmeter.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define VOLTMETER_DUMMY 0

void voltmeter_cfg_setup ( voltmeter_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    // Additional gpio pins

    cfg->an = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

VOLTMETER_RETVAL voltmeter_init ( voltmeter_t *ctx, voltmeter_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return VOLTMETER_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, VOLTMETER_DUMMY );

    // Output pins 

    digital_out_high( &ctx->cs );

    // Input pins

    digital_in_init( &ctx->an, cfg->an );

    return VOLTMETER_OK;
}

void voltmeter_generic_transfer ( voltmeter_t *ctx, spi_master_transfer_data_t *block )
{
    spi_master_start( &ctx->spi );
    spi_master_transfer( &ctx->spi, block );
    spi_master_stop( &ctx->spi );   
}

void voltmeter_write_data ( voltmeter_t *ctx, uint16_t write_data )
{
    spi_master_transfer_data_t block;
    uint8_t buffer[ 2 ];

     buffer[ 1 ] = ( uint8_t ) ( write_data & VOLTMETER_VALUE_5_BIT );
     buffer[ 0 ] = write_data  >> 7;

    block.tx_buffer = buffer;
    block.tx_length = 2;
    block.rx_buffer = 0;
    block.rx_length = 0;

    voltmeter_generic_transfer( ctx, &block ); 
}

uint8_t voltmeter_read_byte ( voltmeter_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t buffer[ 1 ];

    block.tx_buffer = 0;
    block.tx_length = 0;
    block.rx_buffer = buffer;
    block.rx_length = 1;

    voltmeter_generic_transfer ( ctx, &block );
    
    return buffer[ 0 ];
}

uint16_t voltmeter_read_data ( voltmeter_t *ctx )
{
    spi_master_transfer_data_t block;
    uint16_t result;
    uint8_t buffer[ 2 ];

    result = 0;

    block.tx_buffer = 0;
    block.tx_length = 0;
    block.rx_buffer = buffer;
    block.rx_length = 2;

    voltmeter_generic_transfer ( ctx, &block );

    result = buffer[ 0 ];
    result <<= 8;
    result |= buffer[ 1 ];
    result >>= 1;
    result &= VOLTMETER_VALUE_12_BIT;

    return result;
}

void voltmeter_calibration ( voltmeter_t *ctx )
{
    uint16_t read_adc;
    uint8_t cnt;
    float calibration;

    read_adc = 0;
    
    for ( cnt = 0; cnt < 10; cnt++ )
    {
        read_adc = voltmeter_read_data( ctx );
    }
    
    calibration = ( float ) read_adc;
}

float voltmeter_get_voltage ( voltmeter_t *ctx, uint8_t coefficient )
{
    float voltage;
    float sum;
    uint16_t adc_value;
    uint8_t cnt;
    float calibration;
    
    sum = 0;

    for ( cnt = 0; cnt < 10; cnt++ )
    {
        adc_value = voltmeter_read_data( ctx );
        sum += ( float ) adc_value;
        sum -= calibration;
    }

    voltage = sum / 10.0;

    if ( coefficient == VOLTMETER_VCC_5V_COEFF_0 )
    {
        voltage *= VOLTMETER_VCC_5V_COEFF_VAL;
    }
    else if ( coefficient == VOLTMETER_VCC_5V_COEFF_1 )
    {
        voltage *= VOLTMETER_VCC_5V_COEFFICIENT_VAL;
    }
    else if ( coefficient == VOLTMETER_VCC_5V_COEFF_2 )
    {
        voltage *= VOLTMETER_VCC_5V_COEFF_PIC32_VAL;
    }

    return voltage;
}
// ------------------------------------------------------------------------- END

